-- Interface options
atrRunes.GUI = CreateFrame("Frame", "atrRunesGUI", atrRunes)
local GUI = atrRunes.GUI
GUI.name = "atrRunes"
GUI.tempDB = {}
GUI.adjustedSettings = {}

GUI:SetScript("OnShow", function(self)
    GUI.tempDB = {}
    for i, v in pairs(atrRunesDB) do
        GUI.tempDB[i] = v
    end
    
    GUI.adjustedSettings = {}
end)

--
function GUI:AdjustedSetting(key, control)
    GUI.adjustedSettings[key] = control
    
    control.OnPreCancel = function(self)
        atrRunesDB[key] = GUI.tempDB[key]
        if(control.OnCancel) then control.OnCancel(self) end
    end
end

-- Creates a slider
function GUI:CreateSlider(text, parent, min, max, step)
    local name = parent:GetName()..text
    local slider = CreateFrame('Slider', name, parent, 'OptionsSliderTemplate')
    slider:SetMinMaxValues(min, max)
    slider:SetValueStep(step)
    BlizzardOptionsPanel_Slider_Enable(slider)
    
    getglobal(name..'Text'):SetText(text)
    getglobal(name..'Low'):SetText('')
    getglobal(name..'High'):SetText('')
    
    local text = slider:CreateFontString(nil, 'BACKGROUND')
    text:SetFontObject('GameFontHighlightSmall')
    text:SetPoint('LEFT', slider, 'RIGHT', 7, 0)
    slider.valueText = text
    
    return slider
end

-- Creates a check button
function GUI:CreateCheckButton(name, parent)
	local button = CreateFrame('CheckButton', parent:GetName()..name, parent, 'InterfaceOptionsCheckButtonTemplate')
    button.label = getglobal(button:GetName()..'Text')
	button.label:SetText(name)
    button.label:SetWidth(100)
    button.label:SetJustifyH("LEFT")

	return button
end

-- Frame options
function GUI:AddFrameOptions()
    local movable = GUI:CreateCheckButton("Movable", GUI)
	movable:SetScript("OnShow", function(self)
        self:SetChecked(atrRunesDB.movable)
    end)
	movable:SetScript("OnClick", function(self)
        atrRunesDB.movable = self:GetChecked()
        atrRunes:UpdateMoveable()
        GUI:AdjustedSetting("movable", self)
    end)
    movable.OnCancel = function(self)
        atrRunes:UpdateMoveable()
    end
	movable:SetPoint("TOPLEFT", 10, -72)

    local hideOOC = GUI:CreateCheckButton("Hide out of combat", GUI)
    hideOOC:SetScript("OnShow", function(self)
        self:SetChecked(atrRunesDB.hideOOC)
    end)
    hideOOC:SetScript("OnClick", function(self)
        atrRunesDB.hideOOC = self:GetChecked()
        atrRunes:UpdateVisible()
        GUI:AdjustedSetting("hideOOC", self)
    end)
    hideOOC.OnCancel = function(self)
        atrRunes:UpdateVisible()
    end
    hideOOC:SetPoint("LEFT", movable.label, "RIGHT", 10, 0)
    hideOOC:SetPoint("TOP", movable, "TOP", 0, 0)
end

-- Font options
function GUI:AddFontOptions()
    -- size
    local fontSize = GUI:CreateSlider("Font Size", GUI, 8, 36, 1)
    fontSize:SetPoint("LEFT", 10, 0)
    fontSize:SetScript("OnShow", function(self)
        self:SetValue(atrRunesDB.fontSize)
    end)
    fontSize.OnCancel = function(self)
        atrRunes:UpdateFont()
    end
    fontSize:SetScript("OnValueChanged", function(self, value)
		self.valueText:SetText(format('%.1f', value))
        
        atrRunesDB.fontSize = value
        atrRunes:UpdateFont()
        
        GUI:AdjustedSetting("fontSize", self)
	end)
end

-- Build GUI
GUI.title = GUI:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
GUI.title:SetPoint('TOPLEFT', 16, -16)
GUI.title:SetText(select(2, GetAddOnInfo('atrRunes')))

GUI:AddFrameOptions()
GUI:AddFontOptions()

-- Okay/Cancel Buttons
GUI.okay = function (self) end
GUI.cancel = function (self)
    for i, v in pairs(GUI.adjustedSettings) do
        atrRunesDB[i] = GUI.tempDB[i]
        GUI.adjustedSettings[i]:OnPreCancel()
    end
    
    atrRunes:UpdateFont()
end

InterfaceOptions_AddCategory(GUI)

-- Enable a slash command for the addon
SlashCmdList["atrRunes"] = function(msg)
    InterfaceOptionsFrame_OpenToCategory('atrRunes')
end
SLASH_atrRunes1 = "/atrrunes"
SLASH_atrRunes2 = "/atrRunes"